Imports System.ComponentModel
Imports DataLayer
Imports DataLayer.NorthwindDSTableAdapters


<DataObject()> _
Public Class ProductsBL

    Private adapter As New ProductsTableAdapter

    ''' <summary>
    ''' Default SELECT that gets all products
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DataObjectMethod(DataObjectMethodType.Select, True)> _
    Public Function GetProducts() As NorthwindDS.ProductsDataTable
        Return adapter.GetData()
    End Function

    ''' <summary>
    ''' Gets products by the category id
    ''' </summary>
    ''' <param name="categoryId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DataObjectMethod(DataObjectMethodType.Select, False)> _
    Public Function GetProducts(ByVal categoryId As Integer) As NorthwindDS.ProductsDataTable
        Return adapter.GetDataByCategoryId(categoryId)
    End Function

    ''' <summary>
    ''' Gets the product for a specific productid
    ''' </summary>
    ''' <param name="productId"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DataObjectMethod(DataObjectMethodType.Select, False)> _
    Public Function GetProduct(ByVal productId As Integer) As NorthwindDS.ProductsDataTable
        Return adapter.GetDataByProductId(productId)
    End Function


    ''' <summary>
    ''' Updates a product with supplied values
    ''' </summary>
    ''' <param name="ProductName"></param>
    ''' <param name="SupplierID"></param>
    ''' <param name="CategoryID"></param>
    ''' <param name="QuantityPerUnit"></param>
    ''' <param name="UnitPrice"></param>
    ''' <param name="UnitsInStock"></param>
    ''' <param name="UnitsOnOrder"></param>
    ''' <param name="ReorderLevel"></param>
    ''' <param name="Discontinued"></param>
    ''' <param name="ProductID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DataObjectMethod(DataObjectMethodType.Update, True)> _
    Public Function Update( _
                    ByVal ProductName As String, _
                    ByVal SupplierID As System.Nullable(Of Integer), _
                    ByVal CategoryID As System.Nullable(Of Integer), _
                    ByVal QuantityPerUnit As String, _
                    ByVal UnitPrice As System.Nullable(Of Decimal), _
                    ByVal UnitsInStock As System.Nullable(Of Short), _
                    ByVal UnitsOnOrder As System.Nullable(Of Short), _
                    ByVal ReorderLevel As System.Nullable(Of Short), _
                    ByVal Discontinued As Boolean, _
                    ByVal ProductID As Integer) As Boolean

        Dim products As NorthwindDS.ProductsDataTable = adapter.GetDataByProductId(ProductID)
        If (products.Count = 1) Then
            Dim row As NorthwindDS.ProductsRow = products(0)
            row.ProductName = ProductName
            If (SupplierID.HasValue) Then row.SupplierID = SupplierID.Value Else row.SetSupplierIDNull()
            If (CategoryID.HasValue) Then row.CategoryID = CategoryID.Value Else row.SetCategoryIDNull()
            row.QuantityPerUnit = QuantityPerUnit
            If (UnitPrice.HasValue) Then row.UnitPrice = UnitPrice.Value Else row.SetUnitPriceNull()
            If (UnitsInStock.HasValue) Then row.UnitsInStock = UnitsInStock.Value Else row.SetUnitsInStockNull()
            If (UnitsOnOrder.HasValue) Then row.UnitsOnOrder = UnitsOnOrder.Value Else row.SetUnitsOnOrderNull()
            If (ReorderLevel.HasValue) Then row.ReorderLevel = ReorderLevel.Value Else row.SetReorderLevelNull()
            row.Discontinued = Discontinued

            Dim result As Integer = adapter.Update(row)
            If (result = 1) Then
                Return True
            End If
        End If
        Return False

    End Function

    ''' <summary>
    ''' Adds a new product to the system
    ''' </summary>
    ''' <param name="ProductName"></param>
    ''' <param name="SupplierID"></param>
    ''' <param name="CategoryID"></param>
    ''' <param name="QuantityPerUnit"></param>
    ''' <param name="UnitPrice"></param>
    ''' <param name="UnitsInStock"></param>
    ''' <param name="UnitsOnOrder"></param>
    ''' <param name="ReorderLevel"></param>
    ''' <param name="Discontinued"></param>
    ''' <param name="ProductID"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DataObjectMethod(DataObjectMethodType.Insert, True)> _
    Public Function Add( _
                    ByVal ProductName As String, _
                    ByVal SupplierID As System.Nullable(Of Integer), _
                    ByVal CategoryID As System.Nullable(Of Integer), _
                    ByVal QuantityPerUnit As String, _
                    ByVal UnitPrice As System.Nullable(Of Decimal), _
                    ByVal UnitsInStock As System.Nullable(Of Short), _
                    ByVal UnitsOnOrder As System.Nullable(Of Short), _
                    ByVal ReorderLevel As System.Nullable(Of Short), _
                    ByVal Discontinued As Boolean, _
                    ByVal ProductID As Integer) As Boolean

        Dim products As New NorthwindDS.ProductsDataTable
        Dim row As NorthwindDS.ProductsRow = products.NewProductsRow()

        row.ProductName = ProductName
        If (SupplierID.HasValue) Then row.SupplierID = SupplierID.Value Else row.SetSupplierIDNull()
        If (CategoryID.HasValue) Then row.CategoryID = CategoryID.Value Else row.SetCategoryIDNull()
        row.QuantityPerUnit = QuantityPerUnit
        If (UnitPrice.HasValue) Then row.UnitPrice = UnitPrice.Value Else row.SetUnitPriceNull()
        If (UnitsInStock.HasValue) Then row.UnitsInStock = UnitsInStock.Value Else row.SetUnitsInStockNull()
        If (UnitsOnOrder.HasValue) Then row.UnitsOnOrder = UnitsOnOrder.Value Else row.SetUnitsOnOrderNull()
        If (ReorderLevel.HasValue) Then row.ReorderLevel = ReorderLevel.Value Else row.SetReorderLevelNull()
        row.Discontinued = Discontinued

        Dim result As Integer = adapter.Update(row)
        If (result = 1) Then
            Return True
        End If

        Return False

    End Function

    ''' <summary>
    ''' Gets the products but allows sorting and paging
    ''' </summary>
    ''' <param name="sortColumns"></param>
    ''' <param name="startIndex"></param>
    ''' <param name="maxRows"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DataObjectMethod(DataObjectMethodType.Select, False)> _
    Public Function GetProducts(ByVal sortColumns As String, ByVal startIndex As Integer, ByVal maxRows As Integer) As NorthwindDS.ProductsDataTable
        Return adapter.GetProducts(sortColumns, startIndex, maxRows)
    End Function

    ''' <summary>
    ''' Get a count of total products
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetCount() As Integer
        Return adapter.GetCount()
    End Function
End Class
